<?php

declare(strict_types=1);

namespace Drupal\entity_sync_odata\EntitySync\Api;

// Drupal modules.
use Drupal\entity_sync\Exception\InvalidConfigurationException;
// Third-party libraries.
use SaintSystems\OData\Query\Builder;

/**
 * Provides methods for OData API iterator clients.
 */
trait IteratorClientTrait {

  /**
   * Builds and returns the filter query option as expected by OData v4.
   *
   * @param \SaintSystems\OData\Query\Builder $builder
   *   The OData query builder.
   * @param array $filters
   *   The filters passed to the Entity Synchronization client.
   * @param array $options
   *   The options passed to the Entity Synchronization client.
   *
   * @return \SaintSystems\OData\Query\Builder
   *   The updated OData query builder.
   * phpcs:disable
   * @I Support custom filters
   *    type     : improvement
   *    priority : normal
   *    labels   : client
   * phpcs:enable
   */
  protected function buildFilter(
    Builder $builder,
    array $filters,
    array $options
  ): Builder {
    foreach (['created', 'changed'] as $type) {
      if (!(isset($filters["{$type}_start"]) || isset($filters["{$type}_end"]))) {
        continue;
      }

      if (!isset($options["{$type}_field"])) {
        throw new InvalidConfigurationException(sprintf(
          'Filtering by %s time requires the corresponding remote field to be given.',
          $type === 'created' ? 'creation' : 'update'
        ));
      }
    }

    // Add time filters based on creation time.
    // These are not used by the state manager, but they could be passed by the
    // caller in unmanaged list imports.
    if (isset($filters['created_start'])) {
      $builder->where(
        $options['created_field'],
        '>=',
        $this->timestampToIso8601($filters['created_start'])
      );
    }
    if (isset($filters['created_end'])) {
      $builder->where(
        $options['created_field'],
        '<=',
        $this->timestampToIso8601($filters['created_end'])
      );
    }
    // Add time filters based on update time.
    if (isset($filters['changed_start'])) {
      $builder->where(
        $options['changed_field'],
        '>=',
        $this->timestampToIso8601($filters['changed_start'])
      );
    }
    if (isset($filters['changed_end'])) {
      $builder->where(
        $options['changed_field'],
        '<=',
        $this->timestampToIso8601($filters['changed_end'])
      );
    }

    return $builder;
  }

  /**
   * Converts a Unix timestamp to an ISO 8601 string format.
   *
   * @param int $timestamp
   *   The Unix timestamp to convert.
   *
   * @return string
   *   The time in ISO 8601 format.
   */
  protected function timestampToIso8601(int $timestamp): string {
    $datetime = new \DateTime('now', new \DateTimeZone('UTC'));
    $datetime->setTimestamp($timestamp);

    return $datetime->format('Y-m-d\TH:i:s') . 'Z';
  }

}
