<?php

declare(strict_types=1);

namespace Drupal\entity_sync_odata\EntitySync\Api;

// Third-party libraries.
use KrystalCode\Api\Session\SessionManagerInterface;
use KrystalCode\ApiIterator\ClientInterface as IteratorClientInterface;
use KrystalCode\ApiIterator\Iterator as ApiIterator;
use Illuminate\Support\Collection;

/**
 * Iterator client that uses the `$skip/$top` query parameters for paging.
 */
class SkipIteratorClient implements IteratorClientInterface {

  use IteratorClientTrait;
  use ODataClientBuilderTrait;

  /**
   * Constructs a new QueryParametersIteratorClient object.
   *
   * @param \KrystalCode\Api\Session\SessionManagerInterface $sessionManager
   *   The session manager.
   * @param string $entityType
   *   The entity type (resource) of the API client that will be used
   *   e.g. `People`.
   */
  public function __construct(
    protected SessionManagerInterface $sessionManager,
    protected string $entityType
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function list(array $options = [], array $query = []) {
    $options = array_merge(
      [
        'page' => 1,
        'limit' => 100,
        'delay' => NULL,
      ],
      $options
    );
    $query = array_merge(
      [
        'select' => NULL,
        'filter' => NULL,
        'expand' => NULL,
      ],
      $query
    );
    $query['skip'] = ($options['page'] - 1) * $options['limit'];
    $query['top'] = $options['limit'];

    // By default, we return an iterator. Caller code can then loop over it
    // and the iterator will internally handle making the API calls for
    // fetching the objects for each page.
    if (empty($options['bypass_iterator'])) {
      return new ApiIterator(
        $this,
        $options['page'],
        $options['limit'],
        $query,
        FALSE,
        $options['delay']
      );
    }

    // Otherwise, either it is requested to fetch the results directly, or
    // this is a call by the iterator while being looped over. Make the
    // request and return the results.
    unset($options['bypass_iterator']);

    $objects = $this->getList($query);

    // Most commonly, we are not given the total count in the response. The best
    // we can do is to check whether we have less results than the expected per
    // page; that would mean that we are on or past the last page.
    $has_more = NULL;
    $count = $objects->count();
    if ($count === 0 || $count < $options['limit']) {
      $has_more = FALSE;
    }

    return [
      new \CachingIterator(
        $objects->getIterator(),
        \CachingIterator::FULL_CACHE
      ),
      $has_more,
      $query,
    ];
  }

  /**
   * Fetches and returns the objects from the OData API.
   *
   * @param array $query
   *   The query parameters for fetching the objects.
   *
   * @return \Illuminate\Support\Collection
   *   The objects.
   */
  protected function getList(array $query): Collection {
    $builder = $this->getOdataClient()->from($this->entityType);

    if (isset($query['skip'])) {
      $builder->skip($query['skip']);
    }
    if (isset($query['top'])) {
      $builder->take($query['top']);
    }
    if (isset($query['select'])) {
      $builder->select($query['select']);
    }
    if (isset($query['expand'])) {
      $builder->expand($query['expand']);
    }

    $builder = $this->buildFilter(
      $builder,
      $query['filter'][0],
      $query['filter'][1]
    );

    // The Entity Synchronization import manager will be looping over the
    // objects. For efficiency i.e. so that we don't loop over the objects again
    // here, we return the objects unserialized. Synchronizations should use the
    // `serialization` configuration option to instruct the import manager to
    // serialize the objects using the `array_to_object` type and the
    // `jsonSerialize` method.
    return $builder->get();
  }

}
